#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "lcd.h"
#include "evalboard.h"
#include "uart.h"
#include "logo.h"

unsigned char do_animate = 0;
unsigned char address = 0;
unsigned char ticks = 0;
unsigned char leds = 0;
unsigned char x1, x2, y1, y2;
signed char dir1_x = -1;
signed char dir1_y = 1;
signed char dir2_x = -1;
signed char dir2_y = 1;

void animate(void) {
  lcd_drawLine(x1,y1,x2,y2,0);
  if (((x1 + dir1_x) < 0) || ((x1 + dir1_x) > 128))
    dir1_x = -dir1_x;
  if (((x2 + dir2_x) < 0) || ((x2 + dir2_x) > 128))
    dir2_x = -dir2_x;
  if (((y1 + dir1_y) < 0) || ((y1 + dir1_y) > 50))
    dir1_y = -dir1_y;
  if (((y2 + dir2_y) < 0) || ((y2 + dir2_y) > 50))
    dir2_y = -dir2_y;
  x1 += dir1_x; x2 += dir2_x; y1 += dir1_y; y2 += dir2_y;
  lcd_drawLine(x1,y1,x2,y2,1);
}

SIGNAL (SIG_OVERFLOW0) {
//
// Timer0 used for LCD Scrolling Text
//
  char buffer[22];
  address++;
  ticks++;
  eeprom_read_block(&buffer, (void *) (message + address), 22);
  if (buffer[21] == 0x00)
    address = 0x00;
  if (do_animate == 1)
    animate();
  buffer[21] = 0x00;
  lcd_puts_xy(&buffer[0],0,7);
  leds = leds << 1;
  if (leds == 0x08)
    leds = 0x01;
  PORTA = (PORTA & 0xf8) | leds;
}    

void init(void) {
  // Init Interrupts
  lcd_init();
  x1 = 5; x2 = 40;
  y2 = 10; y2 = 30;
  DDRA = 0xff;
  leds = 0x01;  
  TCCR0 = 0x05;                            // Timer0 Prescaler
  TIMSK = TIMSK | 0x01;                    // Timer0 Enable
  uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,XTAL_CPU) );   
  sei();
}


int main(void) {
   unsigned int c;
   init();
   cli();
   lcd_dsp_picture_p(logo, PIC_ELEMENTS);   
   sei();
   while (1) {
     while ((ticks % 10) != 9) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         uart_puts_P("Received: " );
         uart_putc( (unsigned char) c);
         uart_puts_P("\r\n");
       }
     }
     if (ticks == 69) {
	   do_animate = 1;
	   lcd_clrGraphics();
	 }
     uart_puts_P("DELTAWAVE AVR Developer !\r\n");
   }
}
