#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "evalboard.h"
#include "lcd.h"
#include "uart.h"
#include "i2cmaster.h"

unsigned char address = 0;
unsigned char ticks = 0;
unsigned char leds = 0;

SIGNAL (SIG_OVERFLOW0) {
//
// Timer0 used for LCD Scrolling Text
//
  char buffer[21];
  address++;
  ticks++;
  eeprom_read_block(&buffer, (void *) (message + address), 21);
  if (buffer[20] == 0x00)
    address = 0x00;
  buffer[20] = 0x00;
  lcd_gotoxy(0,0);
//  lcd_puts(&deltawave[address % 2][0]);
  lcd_puts(&deltawave[0][0]);
  lcd_gotoxy(0,1);
  lcd_puts(&message3[0]);
  lcd_gotoxy(0,2);
  lcd_puts(&message2[0]);
  lcd_gotoxy(0,4);
  lcd_puts(&buffer[0]);
  leds = leds << 1;
  if (leds == 0x08)
    leds = 0x01;
  PORTA = (PORTA & 0xf8) | leds;
}    

/*
 * uebergebene Zahl i als dezimalzahl anzeigen (target = 0 -> lcd, =1 -> seriell)
 * an der stelle dp wird ein Punkt gesetzt (Float emulation) dp > decimals -> kein Punkt
 * decimals bestimmt, wieviele Stellen angezeigt werden
 * printFormat(12345, 2, 5, 1) gibt 12.345 auf LCD aus
*/
void printFormat(int i, unsigned char dp, unsigned char start, unsigned char decimals, unsigned char target) {
    unsigned char bcd;
    unsigned char j;
    static unsigned int n[] = {10000, 1000, 100, 10, 1};
    if (i < 0) {
      if (target)
        uart_puts_P("-");
      else
        lcd_puts_P("-");
      i = -i;
    }
    for (j = start; j < decimals; j++) {
        bcd = '0';
        while (i >= n[j]) {
            i -= n[j];
            bcd++;
         }
		if (j == dp) {
          if (target)
  		    uart_puts_P(".");
          else
            lcd_puts_P(".");
         }
         if (target)
          uart_putc(bcd);
         else
          lcd_putc(bcd);
      }
}

void init(void) {
  // Init Interrupts
  lcd_init(LCD_DISP_ON);    
  lcd_clrscr();   
  leds = 0x01;
  DDRA = 0xff;
  TCCR0 = 0x05;                            // Timer0 Prescaler
  TIMSK = TIMSK | 0x01;                    // Timer0 Enable
  DDRD = DDRD & 0xE7;                       // Taster als Input
  DDRD = DDRD | 0x04;                       // Relais muss Ausgang sein
  DDRD = DDRD | 0x08;                       // Relais muss Ausgang sein
  uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,XTAL_CPU) );   
  sei();   
}


/*
 * Setzt den Port des PCF8574 IO-Expanders
 * data wird an den Ausgang gelegt
 *
*/
void out_pcf0(unsigned char data) {
        i2c_init();
        i2c_start_wait(pcf8574_0+I2C_WRITE);     // PCF Addressieren
        i2c_write(data);                       // und schreiben
        i2c_stop();
     }

void out_pcf1(unsigned char data) {
        i2c_init();
        i2c_start_wait(pcf8574_1+I2C_WRITE);     // PCF Addressieren
        i2c_write(data);                       // und schreiben
        i2c_stop();
     }

/*
 * ein Byte des 24Cxx an Adresse address auslesen
 *
*/
unsigned char read_24c(unsigned int address) {
        unsigned char ret;
        i2c_init();
        i2c_start_wait(i24c64+I2C_WRITE);     // set device address and write mode
        i2c_write((address >> 8) & 0xff);
        i2c_write(address & 0xff);
        i2c_rep_start(i24c64+I2C_READ);       // set device address and read mode
        ret = i2c_readNak();                    // read one byte         
        i2c_stop();     
        return ret;
}


/*
 * ein Byte des 24Cxx an Adresse address schreiben
 *
*/
void write_24c(unsigned int address, unsigned char data) {
        i2c_init();
        i2c_start_wait(i24c64+I2C_WRITE);     // set device address and write mode
        i2c_write((address >> 8) & 0xff);
        i2c_write(address & 0xff);
        i2c_write(data);
        i2c_stop();
     }

/*
 * Inhalt des 24Cxx als Text seriell downloaden
 * Der Download bricht ab, sobald ein 0xff oder 0x00 im EEprom steht
 *
*/

void downloadEEprom(void) {
        unsigned char ret = 0;
	unsigned long address_count = 0;
        i2c_init();
        i2c_start_wait(i24c64+I2C_WRITE);     // set device address and write mode
        i2c_write(0x00);
        i2c_write(0x00);
        i2c_rep_start(i24c64+I2C_READ);       // set device address and read mode
        do {
          ret = i2c_readAck();                    // read one byte 
          uart_putc(ret);
	  address_count++;
        } while ((ret != 0) && (ret != 0xff) && (address_count < i24cxx_max));
        ret = i2c_readNak();                    // read one byte         
        i2c_stop();     
}

/*
 * Zeichen entweder an UART oder LCD ausgeben
*/
void put_c(unsigned char data, unsigned char target) {
     if (target)
       uart_putc(data);
     else
       lcd_putc(data);
     }


/*
 * Taster an Port D berprfen und ber serielle Schnittstelle Status ausgeben
*/
void checkTaster(void) {
       DDRD = DDRD & 0x8F;                       // Taster als Input
       PORTD = PORTD | 0x70;                     // Pull-Ups
       if ((PIND & 0x10) == 0x00)
         uart_puts_P("Taster 1\r\n");
       if ((PIND & 0x20) == 0x00)
         uart_puts_P("Taster 2\r\n");
       if ((PIND & 0x40) == 0x00)
         uart_puts_P("Taster 3\r\n");
     }

/*
 * Relais seriell steuern kommando r1 -> ein, r0 -> aus
*/
void toggle_relais(void) {
     unsigned char relais = 255;
     unsigned int c;
     ticks = 0;
     while ((ticks < 5) && (relais == 255)) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         relais = c;
         ticks = 0;
       }
     }
     while (ticks < 5) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         if ((c == '0') && (relais == '0')) PORTD = PORTD & 0xfb;
         if ((c == '1') && (relais == '0')) PORTD = PORTD | 0x04;
         if ((c == '0') && (relais == '1')) PORTD = PORTD & 0xf7;
         if ((c == '1') && (relais == '1')) PORTD = PORTD | 0x08;
         ticks = 5;
       }
     }
}

/*
 * PCF IO-Expander seriell steuern
*/
void toggle_pcf(void) {
     unsigned char pcf_no = 255;
     unsigned int c;
     ticks = 0;
     while ((ticks < 5) && (pcf_no == 255)) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         pcf_no = c;
         ticks = 0;
       }
     }

     while (ticks < 5) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         if ((c == '0') && (pcf_no == '0')) out_pcf0(0x00);
         if ((c == '1') && (pcf_no == '0')) out_pcf0(0xff);
         if ((c == '0') && (pcf_no == '1')) out_pcf1(0x00);
         if ((c == '1') && (pcf_no == '1')) out_pcf1(0xff);
         ticks = 5;
       }
     }
}


/*
 * Serielle Daten in EEProm abspeicher, ab Adresse 0x0000
 * TIMEOUT beachten!!
*/
void serialProgram(void) {
     unsigned int c;
     unsigned int adr = 0;
     while (ticks < 10) {
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
         write_24c(adr++, c & 0xff);
         ticks = 0;
       }
     }
     
     }

/*
 * Kleines Demoprogramm fr das MEGA32 System v2
 * Steuerung ber serielle Schnittstelle wie folgt:
             r00 - Relais0 aus
             r01 - Relais0 ein
             r10 - Relais1 aus
             r11 - Relais1 ein
             d  - Inhalt des EEProms (24cxx) seriell downloaden
             p0 - PCF Alles aus
             P1 - PCF Alles ein
             w  - Text, welcher ber serielle Schnittstelle empfangen wird, in EEPROM
                  speichern (10 ticks timeout !!) 
*/
int main(void) {
   unsigned int c;
    init();
   while (1) {
     while (ticks < 10) {
       checkTaster();
       c = uart_getc();
       if (!(c & UART_NO_DATA)) {
          switch (c) {
            case 'r': toggle_relais(); break;
            case 'd': downloadEEprom(); break;
            case 'p': toggle_pcf(); break;            
            case 'w': serialProgram(); break;
          }
       }
     }
     ticks = 0;
     uart_puts_P("DELTAWAVE AVR Developer !\r\n");
   }
}
